@@ -37,6 +37,7 @@ import struct
 import contextlib
 import subprocess
 import shlex
+import io
 
 from setuptools import Command
 from setuptools.sandbox import run_setup
@@ -482,7 +483,7 @@ class easy_install(Command):
         """).lstrip()
 
     def cant_write_to_target(self):
-        msg = self._cant_write_msg % (sys.exc_info()[1], self.install_dir,)
+        msg = self.__cant_write_msg % (sys.exc_info()[1], self.install_dir,)
 
         if not os.path.exists(self.install_dir):
             msg += '\n' + self.__not_exists_id
@@ -1412,13 +1413,8 @@ def extract_wininst_cfg(dist_filename):
             {'version': '', 'target_version': ''})
         try:
             part = f.read(cfglen)
-            # part is in bytes, but we need to read up to the first null
-            # byte.
-            if sys.version_info >= (2, 6):
-                null_byte = bytes([0])
-            else:
-                null_byte = chr(0)
-            config = part.split(null_byte, 1)[0]
+            # Read up to the first null byte.
+            config = part.split(b'\0', 1)[0]
             # Now the config is in bytes, but for RawConfigParser, it should
             #  be text, so decode it.
             config = config.decode(sys.getfilesystemencoding())
@@ -1801,9 +1797,8 @@ def is_python(text, filename='<string>'):
 def is_sh(executable):
     """Determine if the specified executable is a .sh (contains a #! line)"""
     try:
-        fp = open(executable)
-        magic = fp.read(2)
-        fp.close()
+        with io.open(executable, encoding='latin-1') as fp:
+            magic = fp.read(2)
     except (OSError, IOError):
         return executable
     return magic == '#!'
@@ -1845,25 +1840,14 @@ def chmod(path, mode):
 
 
 def fix_jython_executable(executable, options):
-    if sys.platform.startswith('java') and is_sh(executable):
-        # Workaround for Jython is not needed on Linux systems.
-        import java
+    warnings.warn("Use JythonCommandSpec", DeprecationWarning, stacklevel=2)
 
-        if java.lang.System.getProperty("os.name") == "Linux":
-            return executable
+    if not JythonCommandSpec.relevant():
+        return executable
 
-        # Workaround Jython's sys.executable being a .sh (an invalid
-        # shebang line interpreter)
-        if options:
-            # Can't apply the workaround, leave it broken
-            log.warn(
-                "WARNING: Unable to adapt shebang line for Jython,"
-                " the following script is NOT executable\n"
-                "         see http://bugs.jython.org/issue1112 for"
-                " more information.")
-        else:
-            return '/usr/bin/env %s' % executable
-    return executable
+    cmd = CommandSpec.best().from_param(executable)
+    cmd.install_options(options)
+    return cmd.as_header().lstrip('#!').rstrip('\n')
 
 
 class CommandSpec(list):
@@ -1875,6 +1859,13 @@ class CommandSpec(list):
     options = []
     split_args = dict()
 
+    @classmethod
+    def best(cls):
+        """
+        Choose the best CommandSpec class based on environmental conditions.
+        """
+        return cls if not JythonCommandSpec.relevant() else JythonCommandSpec
+
     @classmethod
     def _sys_executable(cls):
         _default = os.path.normpath(sys.executable)
@@ -1897,7 +1888,7 @@ class CommandSpec(list):
 
     @classmethod
     def from_environment(cls):
-        return cls.from_string('"' + cls._sys_executable() + '"')
+        return cls([cls._sys_executable()])
 
     @classmethod
     def from_string(cls, string):
@@ -1906,7 +1897,7 @@ class CommandSpec(list):
         line parseable by shlex.split.
         """
         items = shlex.split(string, **cls.split_args)
-        return JythonCommandSpec.from_string(string) or cls(items)
+        return cls(items)
 
     def install_options(self, script_text):
         self.options = shlex.split(self._extract_options(script_text))
@@ -1942,17 +1933,21 @@ class WindowsCommandSpec(CommandSpec):
 
 class JythonCommandSpec(CommandSpec):
     @classmethod
-    def from_string(cls, string):
-        """
-        On Jython, construct an instance of this class.
-        On platforms other than Jython, return None.
-        """
-        needs_jython_spec = (
+    def relevant(cls):
+        return (
             sys.platform.startswith('java')
             and
             __import__('java').lang.System.getProperty('os.name') != 'Linux'
         )
-        return cls([string]) if needs_jython_spec else None
+
+    @classmethod
+    def from_environment(cls):
+        string = '"' + cls._sys_executable() + '"'
+        return cls.from_string(string)
+
+    @classmethod
+    def from_string(cls, string):
+        return cls([string])
 
     def as_header(self):
         """
@@ -2009,7 +2004,7 @@ class ScriptWriter(object):
         warnings.warn("Use get_header", DeprecationWarning)
         if wininst:
             executable = "python.exe"
-        cmd = cls.command_spec_class.from_param(executable)
+        cmd = cls.command_spec_class.best().from_param(executable)
         cmd.install_options(script_text)
         return cmd.as_header()
 
@@ -2050,7 +2045,7 @@ class ScriptWriter(object):
     @classmethod
     def get_header(cls, script_text="", executable=None):
         """Create a #! line, getting options (if any) from script_text"""
-        cmd = cls.command_spec_class.from_param(executable)
+        cmd = cls.command_spec_class.best().from_param(executable)
         cmd.install_options(script_text)
         return cmd.as_header()
 
